﻿namespace LabsB.Controls.Design
{
    using System;
    using System.Collections;
    using System.ComponentModel;
    using System.ComponentModel.Design;
    using System.Design;
    using System.Drawing;
    using System.Runtime.InteropServices;
    using System.Windows.Forms;
	using System.Windows.Forms.Design;

    public class ActiveDocumentHostDesigner : ControlDesigner
    {
        private DesignerVerb editVerb;
        private bool inheritedReadOnly;
        private MouseHooker mouseHooker;
        private DesignerVerbCollection verbs;

        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);
            if (this.mouseHooker != null)
            {
                this.mouseHooker.Dispose();
                this.mouseHooker = null;
            }
        }

        private bool EatMouseMessage()
        {
            ActiveDocumentHost component = base.Component as ActiveDocumentHost;
            if ((component != null) && (component.ActivationState == ActivationState.Active))
            {
                component.Deactivate();
                return true;
            }
            return this.mouseHooker.HookMouseDown;
        }

        protected override bool GetHitTest(Point point)
        {
            ActiveDocumentHost component = base.Component as ActiveDocumentHost;
            return ((component != null) && (component.ActivationState == ActivationState.Active));
        }

        public override void Initialize(IComponent component)
        {
            base.Initialize(component);
            ActiveDocumentHost host = (ActiveDocumentHost) component;
            host.Loaded += new EventHandler(this.OnActiveDocumentLoaded);
            host.Deactivated += new EventHandler(this.OnActiveDocumentDeactivate);
            host.Activated += new EventHandler(this.OnActiveDocumentActivate);
            base.ShadowProperties["ToolTarget"] = host.ToolTarget;
            base.ShadowProperties["ShowToolbars"] = host.ShowToolbars;
            base.ShadowProperties["ShowMenus"] = host.ShowMenus;
            host.ShowToolbars = false;
            host.ShowMenus = false;
            InheritanceAttribute attribute = (InheritanceAttribute) TypeDescriptor.GetAttributes(component)[typeof(InheritanceAttribute)];
            InheritanceLevel notInherited = InheritanceLevel.NotInherited;
            if (attribute != null)
            {
                notInherited = attribute.InheritanceLevel;
            }
            this.inheritedReadOnly = notInherited == InheritanceLevel.InheritedReadOnly;
        }

        private void OnActiveDocumentActivate(object sender, EventArgs e)
        {
            IComponentChangeService service = (IComponentChangeService) this.GetService(typeof(IComponentChangeService));
            if (service != null)
            {
                service.OnComponentChanging(base.Component, TypeDescriptor.GetProperties(base.Component)["SourceData"]);
            }
            if (this.mouseHooker == null)
            {
                this.mouseHooker = new MouseHooker(this);
            }
            this.mouseHooker.HookMouseDown = true;
			/*
            if ((base.BehaviorService != null) && (base.BehaviorService.AdornerWindowControl != null))
            {
                base.BehaviorService.AdornerWindowControl.Visible = false;
            }*/
        }

        private void OnActiveDocumentDeactivate(object sender, EventArgs e)
        {
            ActiveDocumentHost component = base.Component as ActiveDocumentHost;
            if (component.IsDocumentDataDirty)
            {
                IComponentChangeService service = (IComponentChangeService) this.GetService(typeof(IComponentChangeService));
                if (service != null)
                {
                    service.OnComponentChanged(component, TypeDescriptor.GetProperties(base.Component)["SourceData"], null, null);
                }
            }
            this.mouseHooker.HookMouseDown = false;
			/*
            if ((base.BehaviorService != null) && (base.BehaviorService.AdornerWindowControl != null))
            {
                base.BehaviorService.AdornerWindowControl.Visible = true;
            }*/
        }

        private void OnActiveDocumentLoaded(object sender, EventArgs e)
        {
            if (this.editVerb != null)
            {
                this.editVerb.Enabled = true;
            }
        }

        private void OnVerbEdit(object sender, EventArgs e)
        {
            ActiveDocumentHost component = base.Component as ActiveDocumentHost;
            if ((component != null) && (component.Document != null))
            {
                component.Activate();
            }
        }

        private void OnVerbInsertObject(object sender, EventArgs e)
        {
            IUIService service = (IUIService) this.GetService(typeof(IUIService));
            InsertableObjectPicker form = new InsertableObjectPicker(((ActiveDocumentHost) base.Component).DocumentClassName);
            DialogResult cancel = DialogResult.Cancel;
        Label_002E:
            if (service != null)
            {
                cancel = service.ShowDialog(form);
            }
            else
            {
                cancel = form.ShowDialog(this.Control);
            }
            if (cancel == DialogResult.OK)
            {
                ActiveDocumentHost component = base.Component as ActiveDocumentHost;
                if ((component.Document != null) && component.IsDocumentDataDirty)
                {
                    if (service != null)
                    {
                        cancel = service.ShowMessage(SR.GetString("ActiveDocumentHostReplaceObject"), "ActiveDocumentHost", MessageBoxButtons.OKCancel);
                    }
                    else
                    {
                        cancel = MessageBox.Show(this.Control, SR.GetString("ActiveDocumentHostReplaceObject"), SR.GetString("ActiveDocumentHostReplaceObjectTitle"), MessageBoxButtons.OKCancel, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2, 0);
                    }
                    if (cancel != DialogResult.OK)
                    {
                        return;
                    }
                }
                try
                {
                    if (!form.FileSelected && (((ActiveDocumentHost) base.Component).DocumentClassName != form.ProgId))
                    {
                        TypeDescriptor.GetProperties(base.Component)["DocumentClassName"].SetValue(base.Component, form.ProgId);
                    }
                    else
                    {
                        TypeDescriptor.GetProperties(base.Component)["SourceDocument"].SetValue(base.Component, form.FileName);
                    }
                }
                catch (Exception exception)
                {
                    if (service != null)
                    {
                        service.ShowError(exception);
                    }
                    goto Label_002E;
                }
            }
        }

        protected override void PreFilterProperties(IDictionary properties)
        {
            base.PreFilterProperties(properties);
            string[] strArray = new string[] { "ShowToolbars", "ShowMenus", "ToolTarget" };
            Attribute[] attributes = new Attribute[0];
            for (int i = 0; i < strArray.Length; i++)
            {
                PropertyDescriptor oldPropertyDescriptor = (PropertyDescriptor) properties[strArray[i]];
                if (oldPropertyDescriptor != null)
                {
                    properties[strArray[i]] = TypeDescriptor.CreateProperty(typeof(ActiveDocumentHostDesigner), oldPropertyDescriptor, attributes);
                }
            }
        }

        public override System.Windows.Forms.Design.SelectionRules SelectionRules
        {
            get
            {
                System.Windows.Forms.Design.SelectionRules selectionRules = base.SelectionRules;
                object component = base.Component;
                PropertyDescriptor descriptor = TypeDescriptor.GetProperties(component)["AutoSize"];
                if ((descriptor != null) && ((bool) descriptor.GetValue(component)))
                {
                    selectionRules &= ~System.Windows.Forms.Design.SelectionRules.AllSizeable;
                }
                return selectionRules;
            }
        }

        private bool ShowMenus
        {
            get
            {
                return (bool) base.ShadowProperties["ShowMenus"];
            }
            set
            {
                base.ShadowProperties["ShowMenus"] = value;
            }
        }

        private bool ShowToolbars
        {
            get
            {
                return (bool) base.ShadowProperties["ShowToolbars"];
            }
            set
            {
                base.ShadowProperties["ShowToolbars"] = value;
            }
        }

        private ScrollableControl ToolTarget
        {
            get
            {
                return (ScrollableControl) base.ShadowProperties["ToolTarget"];
            }
            set
            {
                base.ShadowProperties["ToolTarget"] = value;
            }
        }

        public override DesignerVerbCollection Verbs
        {
            get
            {
                if (this.verbs == null)
                {
                    this.editVerb = new DesignerVerb(SR.GetString("ActiveDocumentHostVerbEdit"), new EventHandler(this.OnVerbEdit));
                    this.editVerb.Enabled = ((ActiveDocumentHost) base.Component).Document != null;
                    DesignerVerb verb = new DesignerVerb(SR.GetString("ActiveDocumentHostVerbInsertObject"), new EventHandler(this.OnVerbInsertObject));
                    this.verbs = new DesignerVerbCollection();
                    this.verbs.Add(verb);
                    if (!this.inheritedReadOnly)
                    {
                        this.verbs.Add(this.editVerb);
                    }
                }
                return this.verbs;
            }
        }

        internal class MouseHooker
        {
            private ActiveDocumentHostDesigner designer;
            private IntPtr mouseHookHandle = IntPtr.Zero;
            private GCHandle mouseHookRoot;
            private bool processing;
            internal int thisProcessID;

            public MouseHooker(ActiveDocumentHostDesigner designer)
            {
                this.designer = designer;
            }

            public void Dispose()
            {
                this.UnhookMouse();
            }

            private void HookMouse()
            {
                lock (this)
                {
                    if (this.mouseHookHandle == IntPtr.Zero)
                    {
                        if (this.thisProcessID == 0)
                        {
                            UnsafeNativeMethods.GetWindowThreadProcessId(new HandleRef(this.designer.Control, this.designer.Control.Handle), out this.thisProcessID);
                        }
                        UnsafeNativeMethods.HookProc proc = new UnsafeNativeMethods.HookProc(this.MouseHookProc);
                        this.mouseHookRoot = GCHandle.Alloc(proc);
                        this.mouseHookHandle = UnsafeNativeMethods.SetWindowsHookEx(7, proc, new HandleRef(null, IntPtr.Zero), AppDomain.GetCurrentThreadId());
                    }
                }
            }

            private unsafe IntPtr MouseHookProc(int nCode, IntPtr wparam, IntPtr lparam)
            {
                if (nCode == 0)
                {
                    NativeMethods.MOUSEHOOKSTRUCT* mousehookstructPtr = (NativeMethods.MOUSEHOOKSTRUCT*) lparam;
                    if (mousehookstructPtr != null)
                    {
                        switch (((int) wparam))
                        {
                            case 0x21:
                            case 0xa1:
                            case 0xa4:
                            case 0x204:
                            case 0x207:
                            case 0xa7:
                            case 0x201:
                                if (this.ProcessMouseDown(mousehookstructPtr->hWnd, mousehookstructPtr->pt_x, mousehookstructPtr->pt_y))
                                {
                                    return (IntPtr) 1;
                                }
                                break;
                        }
                    }
                }
                return UnsafeNativeMethods.CallNextHookEx(new HandleRef(this, this.mouseHookHandle), nCode, wparam, lparam);
            }

            private bool ProcessMouseDown(IntPtr hWnd, int x, int y)
            {
                if (!this.processing)
                {
                    IntPtr handle = hWnd;
                    IntPtr ptr2 = this.designer.Control.Handle;
                    Control.FromHandle(handle);
                    if ((handle != ptr2) && !SafeNativeMethods.IsChild(new HandleRef(this, ptr2), new HandleRef(this, handle)))
                    {
                        int num;
                        UnsafeNativeMethods.GetWindowThreadProcessId(new HandleRef(null, handle), out num);
                        if (num != this.thisProcessID)
                        {
                            return false;
                        }
                        try
                        {
                            this.processing = true;
                            return this.designer.EatMouseMessage();
                        }
                        finally
                        {
                            this.processing = false;
                        }
                    }
                }
                return false;
            }

            private void UnhookMouse()
            {
                lock (this)
                {
                    if (this.mouseHookHandle != IntPtr.Zero)
                    {
                        UnsafeNativeMethods.UnhookWindowsHookEx(new HandleRef(this, this.mouseHookHandle));
                        this.mouseHookRoot.Free();
                        this.mouseHookHandle = IntPtr.Zero;
                    }
                }
            }

            public virtual bool HookMouseDown
            {
                get
                {
                    return (this.mouseHookHandle != IntPtr.Zero);
                }
                set
                {
                    if (value)
                    {
                        this.HookMouse();
                    }
                    else
                    {
                        this.UnhookMouse();
                    }
                }
            }
        }
    }
}

