using Microsoft.Win32;
using System;
using System.Collections;
using System.Design;
using System.Drawing;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;
using System.Windows.Forms;
namespace LabsB.Controls.Design
{
	internal class InsertableObjectList : UserControl
	{
		private class ItemInfo
		{
			private Image icon;
			private string iconString;
			private string displayName;
			private string exeName;
			private Guid guid;
			private string progId;
			private static Bitmap DefaultIcon;
			public Image Icon
			{
				get
				{
					if (this.icon == null && this.iconString != null)
					{
						this.LoadIcon();
					}
					return this.icon;
				}
			}
			public string DisplayName
			{
				get
				{
					return this.displayName;
				}
			}
			public string Executable
			{
				get
				{
					return this.exeName;
				}
			}
			public Guid Guid
			{
				get
				{
					return this.guid;
				}
			}
			public string ProgId
			{
				get
				{
					return this.progId;
				}
			}
			static ItemInfo()
			{
				InsertableObjectList.ItemInfo.DefaultIcon = new Bitmap(typeof(InsertableObjectList), "InsertableObject.bmp");
				InsertableObjectList.ItemInfo.DefaultIcon.MakeTransparent();
			}
			public ItemInfo(Guid guid, RegistryKey regKey)
			{
				this.guid = guid;
				this.progId = this.GetSubKeyValue(regKey, "VersionIndependentProgID");
				if (this.progId == null)
				{
					this.progId = this.GetSubKeyValue(regKey, "ProgID");
				}
				this.exeName = this.GetSubKeyValue(regKey, "LocalServer32");
				if (this.exeName == null)
				{
					this.exeName = this.GetSubKeyValue(regKey, "InprocServer32");
				}
				this.displayName = (regKey.GetValue("") as string);
				if (this.displayName == null)
				{
					this.displayName = this.progId;
				}
				this.iconString = this.GetSubKeyValue(regKey, "DefaultIcon");
				if (this.iconString == null)
				{
					this.icon = InsertableObjectList.ItemInfo.DefaultIcon;
				}
			}
			private void LoadIcon()
			{
				if (this.icon == null)
				{
					if (this.iconString != null)
					{
						int num = this.iconString.LastIndexOf(",");
						if (num != -1)
						{
							IntPtr hINSTANCE = Marshal.GetHINSTANCE(typeof(InsertableObjectList.ItemInfo).Module);
							IntPtr intPtr = UnsafeNativeMethods.ExtractIcon(new HandleRef(this, hINSTANCE), this.iconString.Substring(0, num).Trim(), int.Parse(this.iconString.Substring(num + 1).Trim()));
							if (intPtr != IntPtr.Zero)
							{
								this.icon = Bitmap.FromHicon(intPtr);
								UnsafeNativeMethods.DestroyIcon(new HandleRef(this, intPtr));
								return;
							}
							this.icon = InsertableObjectList.ItemInfo.DefaultIcon;
							return;
						}
					}
					else
					{
						this.icon = InsertableObjectList.ItemInfo.DefaultIcon;
					}
				}
			}
			private string GetSubKeyValue(RegistryKey rootKey, string valueName)
			{
				RegistryKey registryKey = rootKey.OpenSubKey(valueName);
				if (registryKey != null)
				{
					try
					{
						return registryKey.GetValue(null) as string;
					}
					finally
					{
						registryKey.Close();
					}
				}
				return null;
			}
			public override string ToString()
			{
				return this.DisplayName;
			}
		}
		private const int ItemDrawSpace = 5;
		private ListBox lbItems;
		private ArrayList insertableItems;
		private int lastSelected = -1;
		private Guid selectedGuid;
		private string selectedProgId;
		public event EventHandler ItemSelected;
		public Guid SelectedItemGuid
		{
			get
			{
				if (this.lbItems.SelectedIndex != -1)
				{
					return ((InsertableObjectList.ItemInfo)this.lbItems.SelectedItem).Guid;
				}
				return this.selectedGuid;
			}
			set
			{
				if (this.lbItems != null && this.lbItems.Items.Count > 0)
				{
					for (int i = 0; i < this.lbItems.Items.Count; i++)
					{
						InsertableObjectList.ItemInfo itemInfo = (InsertableObjectList.ItemInfo)this.lbItems.Items[i];
						if (itemInfo != null && itemInfo.Guid == value)
						{
							this.lbItems.SelectedIndex = i;
							return;
						}
					}
				}
				this.selectedGuid = value;
			}
		}
		public string SelectedItemProgId
		{
			get
			{
				if (this.lbItems.SelectedIndex != -1)
				{
					return ((InsertableObjectList.ItemInfo)this.lbItems.SelectedItem).ProgId;
				}
				return this.selectedProgId;
			}
			set
			{
				if (this.lbItems != null && this.lbItems.Items.Count > 0)
				{
					for (int i = 0; i < this.lbItems.Items.Count; i++)
					{
						InsertableObjectList.ItemInfo itemInfo = (InsertableObjectList.ItemInfo)this.lbItems.Items[i];
						if (itemInfo != null && itemInfo.ProgId == value)
						{
							this.lbItems.SelectedIndex = i;
							return;
						}
					}
				}
				this.selectedProgId = value;
			}
		}
		public InsertableObjectList()
		{
			this.InitializeComponent();
		}
		private void InitializeComponent()
		{
			this.lbItems = new ListBox();
			base.SuspendLayout();
			this.lbItems.Dock = DockStyle.Fill;
			this.lbItems.DrawMode = DrawMode.OwnerDrawFixed;
			this.lbItems.IntegralHeight = false;
			this.lbItems.ItemHeight = 32;
			this.lbItems.Name = "lbItems";
			this.lbItems.Size = new Size(376, 424);
			this.lbItems.TabIndex = 0;
			this.lbItems.KeyDown += new KeyEventHandler(this.lbItems_KeyDown);
			this.lbItems.Resize += new EventHandler(this.lbItems_Resize);
			this.lbItems.DoubleClick += new EventHandler(this.lbItems_DoubleClick);
			this.lbItems.DrawItem += new DrawItemEventHandler(this.lbItems_DrawItem);
			this.lbItems.SelectedIndexChanged += new EventHandler(this.lbItems_SelectedIndexChanged);
			base.Controls.AddRange(new Control[]
			{
				this.lbItems
			});
			base.Name = "InsertableObjectList";
			base.Size = new Size(376, 424);
			base.EnabledChanged += new EventHandler(this.InsertableObjectList_EnabledChanged);
			base.Load += new EventHandler(this.InsertableObjectList_Load);
			base.ResumeLayout(false);
		}
		private void InsertableObjectList_Load(object sender, EventArgs e)
		{
			if (this.insertableItems != null || base.DesignMode)
			{
				return;
			}
			int num = -1;
			this.insertableItems = new ArrayList();
			RegistryKey registryKey = Registry.ClassesRoot.OpenSubKey("CLSID");
			try
			{
				string[] subKeyNames = registryKey.GetSubKeyNames();
				for (int i = 0; i < subKeyNames.Length; i++)
				{
					string text = subKeyNames[i];
					RegistryKey registryKey2 = registryKey.OpenSubKey(text);
					try
					{
						RegistryKey registryKey3 = registryKey2.OpenSubKey("Insertable");
						RegistryKey registryKey4 = registryKey2.OpenSubKey("NotInsertable");
						RegistryKey registryKey5 = registryKey2.OpenSubKey("Control");
						try
						{
							if (registryKey4 == null && registryKey3 != null && registryKey5 == null)
							{
								RegistryKey registryKey6 = registryKey2.OpenSubKey("ProgID");
								if (registryKey6 != null)
								{
									try
									{
										string text2 = (string)registryKey6.GetValue(null);
										if (text2 != null)
										{
											RegistryKey registryKey7 = Registry.ClassesRoot.OpenSubKey(text2 + "\\NotInsertable");
											if (registryKey7 == null)
											{
												Guid guid = new Guid(text);
												InsertableObjectList.ItemInfo itemInfo = new InsertableObjectList.ItemInfo(guid, registryKey2);
												if ((this.selectedGuid != Guid.Empty && guid == this.selectedGuid) || (this.selectedProgId != null && this.selectedProgId == itemInfo.ProgId))
												{
													num = this.insertableItems.Count;
												}
												this.insertableItems.Add(itemInfo);
											}
											else
											{
												registryKey7.Close();
											}
										}
									}
									finally
									{
										registryKey6.Close();
									}
								}
							}
						}
						finally
						{
							if (registryKey3 != null)
							{
								registryKey3.Close();
							}
							if (registryKey4 != null)
							{
								registryKey4.Close();
							}
							if (registryKey5 != null)
							{
								registryKey5.Close();
							}
						}
					}
					finally
					{
						registryKey2.Close();
					}
				}
			}
			finally
			{
				registryKey.Close();
			}
			this.lbItems.DataSource = this.insertableItems;
			if (num != -1)
			{
				this.lbItems.SelectedIndex = num;
			}
		}
		private void lbItems_DrawItem(object sender, DrawItemEventArgs e)
		{
			if (e.Index < 0)
			{
				return;
			}
			bool flag = (e.State & DrawItemState.Selected) == DrawItemState.Selected;
			e.DrawBackground();
			InsertableObjectList.ItemInfo itemInfo = (InsertableObjectList.ItemInfo)this.lbItems.Items[e.Index];
			Rectangle bounds = e.Bounds;
			int height = bounds.Height;
			if (itemInfo.Icon != null)
			{
				Graphics arg_81_0 = e.Graphics;
				Image arg_81_1 = itemInfo.Icon;
				bounds = e.Bounds;
				int arg_7C_0 = bounds.X;
				bounds = e.Bounds;
				arg_81_0.DrawImage(arg_81_1, new Rectangle(arg_7C_0, bounds.Y, height, height));
			}
			StringFormat stringFormat = new StringFormat();
			stringFormat.Alignment = 0;
			stringFormat.LineAlignment = StringAlignment.Center;
			Brush brush = null;
			if (!base.Enabled)
			{
				brush = SystemBrushes.ControlDark;
			}
			else
			{
				if (flag)
				{
					brush = SystemBrushes.HighlightText;
				}
				else
				{
					brush = SystemBrushes.WindowText;
				}
			}
			Graphics arg_109_0 = e.Graphics;
			string arg_109_1 = itemInfo.DisplayName;
			Font arg_109_2 = this.lbItems.Font;
			Brush arg_109_3 = brush;
			int arg_FE_0 = 5 + height;
			bounds = e.Bounds;
			int arg_FE_1 = bounds.Y;
			bounds = e.Bounds;
			arg_109_0.DrawString(arg_109_1, arg_109_2, arg_109_3, new Rectangle(arg_FE_0, arg_FE_1, bounds.Width - height, height), stringFormat);
			stringFormat.Dispose();
			if (flag)
			{
				e.DrawFocusRectangle();
			}
		}
		private void lbItems_SelectedIndexChanged(object sender, EventArgs e)
		{
			if (this.lastSelected != -1)
			{
				this.lbItems.Invalidate(this.lbItems.GetItemRectangle(this.lastSelected));
			}
			this.lastSelected = this.lbItems.SelectedIndex;
		}
		private void InsertableObjectList_EnabledChanged(object sender, EventArgs e)
		{
			this.lbItems.Invalidate();
		}
		private void lbItems_Resize(object sender, EventArgs e)
		{
			this.lbItems.Invalidate();
		}
		private void lbItems_DoubleClick(object sender, EventArgs e)
		{
			if (this.ItemSelected != null && this.lbItems.SelectedIndex != -1)
			{
				this.ItemSelected.Invoke(this, e);
			}
		}
		private void lbItems_KeyDown(object sender, KeyEventArgs e)
		{
			if (e.KeyCode == Keys.Return)
			{
				e.Handled = true;
				this.lbItems_DoubleClick(sender, e);
			}
		}
		public void RefreshItems()
		{
			this.InsertableObjectList_Load(null, EventArgs.Empty);
		}
	}
}
